#ifndef UDGPERPENDICULARDISTANCETOOL_H
#define UDGPERPENDICULARDISTANCETOOL_H

#include "genericdistancetool.h"

namespace udg {

/**
    Eina per fer mesures TA-GT.

    El funcionament s el segent:
    - Primer es dibuixa una lnia.
    - Un cop dibuixada la lnia es traa una lnia perpendicular a la primera lnia segons la posici del mouse.
    - Un cop acabada la segona lnia, es traa una tercera lnia parallela a l'anterior.
    - Un cop fixada la lnia parallela, apareix la distncia entre les lnies paralleles.

    Per marcar cadascun dels punts es far amb un clic o doble-clic amb el bot esquerre del ratol.
    Un cop marcat el primer punt, es mostrar una lnia que unir el primer punt marcat amb el punt on es trobi el punter del ratol.
    Un cop marcat el segon punt, es mostrar una lnia perpendicular a la primera que unir aquesta amb el punt on es trobi el punter del ratol.
    Un cop marcat el tercer punt, es mostrar una lnia perpendicular a la primera i parallela a la segona que unir aquesta amb el punt on es trobi el punter
    del ratol.
    Mentre l'usuari no hagi marcat l'ltim punt, no es mostrar cap informaci respecte a la distncia en aquell moment.

    Un cop marcat l'ltim punt, la mesura es mostrar en millmetres si existeix informaci d'espaiat i en pxels altrament.

    Un cop acabada la mesura, es poden annotar successivament tantes mesures com es desitgi.

    Les mesures anotades noms apareixeran sobre les imatges sobre les que s'han realitzat, per no s'esborren si canviem d'imatge.
    Si canviem d'imatge per tornem a mostrar-la ms endavant les anotacions fetes sobre aquesta anteriorment tornaran a aparixer.

    Quan es canvi l'input del visor, les anotacions fetes fins aquell moment s'esborraran.
    Quan es desactivi l'eina, les anotacions fetes fins aquell moment es mantindran.

    TODO Canviar el nom de la classe.
  */
class PerpendicularDistanceTool : public GenericDistanceTool {

Q_OBJECT

public:

    PerpendicularDistanceTool(QViewer *viewer, QObject *parent = 0);
    ~PerpendicularDistanceTool();

protected:
    void handleLeftButtonPress();
    void handleMouseMove();
    void abortDrawing();

private:

    /// Possibles estats de l'eina.
    enum State { NotDrawing, DrawingReferenceLine, DrawingFirstPerpendicularLine, DrawingSecondPerpendicularLine };

private:

    /// Decideix qu s'ha de fer quan es rep un clic.
    void handleClick();
    /// Afegeix el primer punt, per comenar a dibuixar la lnia de referncia.
    void addFirstPoint();
    /// Afegeix el segon punt, per acabar de dibuixar la lnia de referncia i comenar la primera perpendicular.
    void addSecondPoint();
    /// Afegeix el tercer punt, per acabar de dibuixar la primera lnia perpendicular i comenar la segona.
    void addThirdPoint();
    /// Afegeix el quart i ltim punt, per acabar de dibuixar la segona lnia perpendicular i calcular la distncia.
    void addFourthPoint();
    /// Actualitza la lnia de referncia perqu vagi del primer punt a la posici actual del ratol.
    void updateReferenceLine();
    /// Actualitza la primera lnia perpendicular perqu sigui perpendicular a la lnia de referncia i vagi des d'aquesta a la posici actual del ratol (o al
    /// punt ms proper de manera que encara toqui amb la lnia de referncia).
    void updateFirstPerpendicularLine();
    /// Actualitza la segona lnia perpendicular perqu sigui perpendicular a la lnia de referncia i pel mateix costat que la primera lnia perpendicular i
    /// vagi des d'aquesta a la posici actual del ratol (o al punt ms proper de manera que encara toqui amb la lnia de referncia).
    void updateSecondPerpendicularLine();
    /// Actualitza la lnia de referncia i refresca la visualitzaci.
    void updateReferenceLineAndRender();
    /// Actualitza la primera lnia perpendicular i refresca la visualitzaci.
    void updateFirstPerpendicularLineAndRender();
    /// Actualitza la segona lnia perpendicular i refresca la visualitzaci.
    void updateSecondPerpendicularLineAndRender();
    /// Calcula la distncia i la dibuixa.
    void drawDistance();
    /// Dibuixa la lnia de distncia.
    void drawDistanceLine();

    /// Equalitza la profunditat dels elements que formen l'anotaci TA-GT final.
    void equalizeDepth();

private slots:

    /// Torna l'eina al seu estat inicial.
    void reset();

private:

    /// La lnia de referncia, per fer desprs dues lnies perpendiculars a aquesta.
    QPointer<DrawerLine> m_referenceLine;
    /// La primera lnia perpendicular a la de referncia, que s un extrem de la distncia.
    QPointer<DrawerLine> m_firstPerpendicularLine;
    /// La segona lnia perpendicular a la de referncia, que s l'altre extrem de la distncia.
    QPointer<DrawerLine> m_secondPerpendicularLine;

    /// Estat de l'eina.
    State m_state;

};

}

#endif
