#include "zoomtool.h"
#include "qviewer.h"
#include "logging.h"
#include "q3dviewer.h"
// Vtk
#include <vtkRenderer.h>
#include <vtkRenderWindow.h>
#include <vtkRenderWindowInteractor.h>
#include <vtkCommand.h>

namespace udg {

ZoomTool::ZoomTool(QViewer *viewer, QObject *parent)
 : Tool(viewer, parent)
{
    m_state = None;
    m_toolName = "ZoomTool";
    // Ens assegurem que desde la creació tenim un viewer vàlid
    Q_ASSERT(m_viewer);
    // TODO This could be done better if we had some method that returns the type of the viewer
    if (Q3DViewer::castFromQViewer(m_viewer))
    {
        m_mustRenderOnEnd = true;
    }
    else
    {
        m_mustRenderOnEnd = false;
    }
}

ZoomTool::~ZoomTool()
{
    m_viewer->unsetCursor();
}

void ZoomTool::handleEvent(unsigned long eventID)
{
    switch (eventID)
    {
        case vtkCommand::LeftButtonPressEvent:
            this->startZoom();
        break;

        case vtkCommand::MouseMoveEvent:
            this->doZoom();
        break;

        case vtkCommand::LeftButtonReleaseEvent:
            this->endZoom();
        break;

        default:
        break;
    }
}

void ZoomTool::startZoom()
{
    m_state = Zooming;
    m_viewer->getInteractor()->GetRenderWindow()->SetDesiredUpdateRate(m_viewer->getInteractor()->GetDesiredUpdateRate());
}

void ZoomTool::doZoom()
{
    if (m_state == Zooming)
    {
        m_viewer->setCursor(QCursor(QPixmap(":/images/zoom.png")));
        double *center = m_viewer->getRenderer()->GetCenter();
        int dy = m_viewer->getEventPosition().y() - m_viewer->getLastEventPosition().y();
        // TODO el 10.0 és un valor constant que podria refinar-se si es volgués (motion factor)
        double dyf = 10.0 * (double)(dy) / (double)(center[1]);
        m_viewer->zoom(pow((double)1.1, dyf));
    }
}

void ZoomTool::endZoom()
{
    if (m_state == Zooming)
    {
        m_viewer->unsetCursor();
        m_state = None;
        m_viewer->getInteractor()->GetRenderWindow()->SetDesiredUpdateRate(m_viewer->getInteractor()->GetStillUpdateRate());

        if (m_mustRenderOnEnd)
        {
            // Necessari perquè es torni a renderitzar a alta resolució en el 3D
            m_viewer->render();
        }
    }
}

}
